"use strict";

/* BEGIN LICENSE BLOCK

for detail, please refer to license.txt in the root folder of this extension

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

If you use large portions of the code please attribute to the authors
(Axel Grude)

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You can download a copy of the GNU General Public License at
http://www.gnu.org/licenses/gpl-3.0.txt or get a free printed
copy by writing to:
  Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.

END LICENSE BLOCK
*/


// note: in QuickFolder_s, this object is simply called "Filter"!
quickFilters.List = {
  eventsAreHooked: false ,

  // FILTER LIST DIALOG FUNCTIONS
  getFilterListElement: function()
  {
    var el = document.getElementById("filterList");
    if (!el)
      el = document.getElementById("filterTree");
    return el;
  } ,

  getSelectedFilterAt: function(list, i)
  {
    if (typeof list.selectedItems !== "undefined")
      return list.selectedItems[i]._filter;

    // SeaMonkey uses a tree view: - we ignore i as we know that there is only 1 item selected, hence i would be selection index 0
    var start = new Object();
    var end = new Object();

      list.view.selection.getRangeAt(0,start,end);

    // return list.view.getFilterAt(start.value);
    return getFilter(start.value); // defined in FilterListDialog.js (SM only)


  } ,

  getSelectedCount: function(list)
  {
    if (typeof list.selectedItems !== "undefined")
      return list.selectedItems.length;
    return list.view.selection.count;
  } ,

  onTop : function (evt)
  {
    var filtersList = this.getFilterList(); // Tb / SM
    var list = this.getFilterListElement();
    try {
      if (this.getSelectedCount(list) != 1) {
        alert("Exactly one filter entry must be highlighted!");
        return;
      }
      var activeFilter = this.getSelectedFilterAt(list, 0);
      if (activeFilter) {
        filtersList.removeFilter(activeFilter);
        filtersList.insertFilterAt(0, activeFilter);
        this.rebuildFilterList();  // not on SM?
        // SM
        if (list.view) {
          list.view.selection.clearSelection();  // Pb: gFilterTree
          list.view.selection.select(0);
          list.focus();
        }
        else
          this.onFindFilter(false);

        document.getElementById("quickFilters-reorderButtonTop").disabled=true;
      }
    }
    catch(ex) {
      quickFilters.Util.logException('Exception while moving filter to top: ', ex);
    }
  } ,

  onBottom : function (evt)
  {
    var filtersList = this.getFilterList();
    var list =this.getFilterListElement();
    try {
      if (this.getSelectedCount(list) != 1) {
        alert("Exactly one filter entry must be highlighted!");
        return;
      }
      var activeFilter = this.getSelectedFilterAt(list, 0);
      if (activeFilter) {
        filtersList.removeFilter(activeFilter);
        filtersList.insertFilterAt(filtersList.filterCount, activeFilter); // rolled back :P
        this.rebuildFilterList();
        // SM
        if (list.view) {
          list.view.selection.clearSelection();  // Pb: gFilterTree
          list.view.selection.select(filtersList.filterCount-1);
          list.focus();
        }
        else
          this.onFindFilter(false);

        document.getElementById("quickFilters-reorderButtonBottom").disabled=true;
      }
    }
    catch(ex) {
      quickFilters.Util.logException('Exception while moving filter to bottom: ', ex);
    }
  } ,

  onUp: function(event) {
    var searchBox = document.getElementById("quickFilters-Search");
    if (searchBox.value) {
      var filtersList = this.getFilterList(); // Tb / SM
      var list = this.getFilterListElement();
      if (this.getSelectedCount(list) != 1)
        return;

      var activeFilter = this.getSelectedFilterAt(list, 0);
      if (activeFilter) try {
        var nextIndex = list.selectedIndex-1;
        var nextFilter = list.getItemAtIndex(nextIndex)._filter;
        this.rebuildFilterList();

        // assumption: item stays selected even after removing the search condition
        var newIndex = list.selectedIndex-1;
        filtersList.removeFilter(activeFilter);

        // insert before next visible item
        // go up from selected index until finding the correct filter name
        while (nextFilter.filterName!=list.getItemAtIndex(newIndex)._filter.filterName && nextIndex<list.itemCount)
          newIndex--;
        filtersList.insertFilterAt(newIndex, activeFilter);
        this.rebuildFilterList();
        list.selectedIndex = newIndex;

      }
      catch (ex) {
        quickFilters.Util.logException('quickFilters.List.onDown: ', ex);
      }
      this.onFindFilter(false);
    }
    else
      moveCurrentFilter(Components.interfaces.nsMsgFilterMotion.up);
  } ,

  onDown: function(event) {
    var searchBox = document.getElementById("quickFilters-Search");
    if (searchBox.value) {
      var filtersList = this.getFilterList(); // Tb / SM
      var list = this.getFilterListElement();
      if (this.getSelectedCount(list) != 1)
        return;

      var activeFilter = this.getSelectedFilterAt(list, 0);
      if (activeFilter) try {
        var nextIndex = list.selectedIndex+1;
        var nextFilter = list.getItemAtIndex(nextIndex)._filter;
        this.rebuildFilterList();

        // assumption: item stays selected even after removing the search condition
        var newIndex = list.selectedIndex+1;
        filtersList.removeFilter(activeFilter);


        // insert after next visible item
        // go down from selected index until finding the correct filter name
        while (nextFilter.filterName!=list.getItemAtIndex(newIndex)._filter.filterName && nextIndex<list.itemCount)
          newIndex++;
        filtersList.insertFilterAt(newIndex, activeFilter);
        this.rebuildFilterList();
        list.selectedIndex = newIndex;

      }
      catch (ex) {
        quickFilters.Util.logException('quickFilters.List.onDown: ', ex);
      }
      this.onFindFilter(false);
    }
    else
      moveCurrentFilter(Components.interfaces.nsMsgFilterMotion.down);
  } ,

  getListElementCount: function (list) {
    if (typeof list.getRowCount !== "undefined")
      return list.getRowCount();
    return list.view.rowCount; // SM: treeview
  } ,

  onSelectFilter : function (evt) {
    var list = this.getFilterListElement();
    var numFiltersSelected = this.getSelectedCount(list);
    var oneFilterSelected = (numFiltersSelected === 1);
    var buttonTop = document.getElementById("quickFilters-reorderButtonTop");
    var buttonBottom = document.getElementById("quickFilters-reorderButtonBottom");
    var upDisabled = !(oneFilterSelected &&
                       this.getSelectedFilterAt(list, 0) != list.childNodes[1]);
    if (list.currentIndex === 0) // SM
      upDisabled = true;
    buttonTop.disabled = upDisabled;
    var downDisabled = (!oneFilterSelected
        || list.currentIndex === this.getListElementCount(list)-1);
    buttonBottom.disabled = downDisabled;
  } ,

  onLoadFilterList: function(evt)
  {
    function removeElement(el) {
      el.collapsed = true;
    }
    function formatListLabel(el) {
      if (el) {
        el.setAttribute('crop','end');
        el.minwidth=15;
      }
    }

    // check whether [Bug 450302] has landed
    let nativeSearchBox = document.getElementById("searchBox");
    // check whether QuickFolder_s already does these modifications
    let quickFolderSearchBox = document.getElementById("qf-Filter");
    //move buttons to the correct place
    var buttonBottom = document.getElementById("quickFilters-reorderButtonBottom");
    var buttonTop = document.getElementById("quickFilters-reorderButtonTop");
    var down = document.getElementById("reorderDownButton");
    var up = document.getElementById("reorderUpButton");
    var searchBox = document.getElementById("quickFilters-Search");
    var countBox = document.getElementById("quickFilters-Count");
    if (nativeSearchBox || quickFolderSearchBox) {
      // once [Bug 450302] has landed, we do not need to add this functionality ourselves :)
      // also same functionality is provided by QuickFolder_s already, so no need to do it from here
      removeElement(buttonTop);
      removeElement(buttonBottom);
      removeElement(searchBox);
      removeElement(countBox);
      // in this case, this id should be assigned already by the bugfix
      formatListLabel(document.getElementById("filterListLabel"));
      return;
    }
    // DOMi ugliness.
    if (up){
      up.parentNode.insertBefore(buttonTop, up);
    }
    if (down){
      down.parentNode.insertBefore(buttonBottom, down.nextSibling);
    }
    // overwrite handlers for moving filters while search is active
    // add additional listener for the filter list to select event
    var filterList = this.getFilterListElement();
    if (filterList) {
      filterList.addEventListener("select",
        function(e) { quickFilters.List.onSelectFilter(e);},
        false);
      // make sure to disable the correct buttons on dialog load
      // the delay times are picked somewhat arbitrarily, sorry.
      window.setTimeout(function() {quickFilters.List.onSelectFilter(null);}, 250);

      // Update filter counts after new and delete:
      // removed DOM_NodeInserted events and chose some monkey patching instead.
      if (!quickFilters.List.eventsAreHooked) {
        let theOnNew = onNewFilter;
        if (theOnNew) {
          onNewFilter = function() {
            theOnNew(arguments);
            try {
              quickFilters.List.updateCountBox();
            }
            catch(e) {
              if (quickFilters && quickFilters.Util)
                quickFilters.Util.logException('onNewFilter - ',e);
            }
          }
        }
        let theOnDelete = onDeleteFilter;
        if (theOnDelete) {
          onDeleteFilter = function() {
            theOnDelete(arguments);
            try {
              quickFilters.List.updateCountBox();
            }
            catch(e) {
              if (quickFilters && quickFilters.Util)
                quickFilters.Util.logException('onDeleteFilter - ',e);
            }
          }
        }
        quickFilters.List.eventsAreHooked = true; // avoid multiple hooking.
      }

    }

    // the following changes to the dialog layout are fairly fundamental, but they follow the (UI-reviewed)
    // modifications in  [Bug 450302].
    // 1. [Run Now] is relocated to the bottom, between filter target drop down and Filter Log.
    //    there is a certain functional logic to this order
    // 2. The search {filters} box is inserted to the right of the servers dropdown.
    //    Again this makes sense as both elements cause the list contents to change.
    // 3. An item count is appended to the right of the list description label
    //    TO DO: description should be collapsible in favor of item count.
    if (quickFilters.Util.Application === 'Thunderbird') {
      // move the search filter box
      let dropDown = document.getElementById("serverMenu");

      dropDown.parentNode.insertBefore(searchBox, dropDown.nextSibling);
      dropDown.addEventListener("command", function(e) { window.setTimeout(function() {quickFilters.List.onFindFilter(false);}, 50); }, false);

      // create a container that holds list label and count...
      // more DOMi ugliness...
      let rowAbove = filterList.parentNode.parentNode.previousSibling;
      let filterListLabel = rowAbove.firstChild;
      filterListLabel.id='filterListLabel';
      formatListLabel(filterListLabel);

      let hbox = document.createElement('hbox');
      rowAbove.appendChild(hbox);
      hbox.appendChild(filterListLabel);
      let spc = document.createElement('spacer');
      spc.flex = 1;
      hbox.appendChild(spc);
      // countBox.flex="1"; // make sure this is never obscured by the label
      hbox.appendChild(countBox);

      this.updateCountBox();


      // we need to overwrite the existing functions in order to support the "filtered" state
      // REVIEWERS PLEASE NOTE: this is to replace the existing functionality
      //                        in order to supporte moving filters
      //                        while the search mode is active
      var reorderUpButton = document.getElementById("reorderUpButton");
      reorderUpButton.setAttribute("oncommand", "quickFilters.List.onUp(event);");
      var reorderDownButton = document.getElementById("reorderDownButton");
      reorderDownButton.setAttribute("oncommand", "quickFilters.List.onDown(event);");

      var runFiltersButton =  document.getElementById("runFiltersButton");
      // find the log button (first button in hbox) and move it down
      var filterLogButton = dropDown.parentNode.getElementsByTagName("button")[0];
      // insert Filter log button at the bottom
      runFiltersButton.parentNode.insertBefore(filterLogButton, runFiltersButton)
      // move run filters button to left
      var runFiltersFolderMenu =  document.getElementById("runFiltersFolder");
      runFiltersFolderMenu.parentNode.appendChild(runFiltersButton);
    }
    else {
      // for the moment we do not support this on SM / POstbox as I have problems with removing stuff from the treeview!
      // in future we need to build our own tree
      // build a treeview that supports hidden elements and overwrite gFilterTreeView
      // #maildev@Neil: could create a virtual list, the SeaMonkey view is only interested in the filterCount property and the getFilterAt method
      searchBox.collapsed = true;
    }

  } ,
  
  onCopy: function() {
    ;  
  } ,

  onPaste: function() {
    ;  
  } ,

  // helper function for SeaMonkey/Postbox (which doesn't have a gCurrentFilterList)
  getFilterList: function()
  {
    try {
      if (typeof gCurrentFilterList !== "undefined")
        return gCurrentFilterList;
      if (currentFilterList)
        return currentFilterList();
    }
    catch(ex) {
      quickFilters.Util.logException('quickFilters.List.getFilterList: ', ex);
    }
    return null;
  } ,

  // SeaMonkey / Postbox helper
  gFilterTreeView: function() {
    if (typeof gFilterTreeView !== "undefined")
      return gFilterTreeView; // SM
    //Postbox
    return gFilterTree.view;

  } ,

  updateCountBox: function()
  {
    var countBox = document.getElementById("quickFilters-Count");
    var sum = this.getFilterList().filterCount;
    var filterList = this.getFilterListElement();
    var len = this.getListElementCount(filterList);

    if (len === sum)
      countBox.value =
        (len === 1)
        ? document.getElementById ('quickFilters-Count-1-item').value
        : len.toString() + " " + document.getElementById ('quickFilters-Count-items').value;
    else
      countBox.value = document.getElementById ('quickFilters-Count-n-of-m').value
        .replace('{0}', len.toString())
        .replace('{1}', sum.toString());

  } ,

  rebuildFilterList: function()
  {
    if (typeof gCurrentFilterList !== "undefined") {
      rebuildFilterList(gCurrentFilterList);
    }
    else {
      if (quickFilters.Util.Application === 'Postbox') {
        refresh();
        this.updateCountBox();
        return;
      }

      // force a repaint through the BoxObject
      var fl = this.getFilterListElement();

      // from: SM's setServer(uri) function
      var resource = gRDF.GetResource(gCurrentServerURI);
      var msgFolder = resource.QueryInterface(Components.interfaces.nsIMsgFolder);

      //Calling getFilterList will detect any errors in rules.dat, backup the file, and alert the user
      switch(quickFilters.Util.Application) {
        case 'Postbox':
          this.gFilterTreeView().filterList = msgFolder.getFilterList(gFilterListMsgWindow);
          break;
        default:
          this.gFilterTreeView().filterList = msgFolder.getEditableFilterList(gFilterListMsgWindow);
          break;
      }
      fl.boxObject.invalidate();
    }
    this.updateCountBox();
  } ,

/**
 * Called when the search button is clicked, this will narrow down the amount
 * of filters displayed in the list, using the search term to filter the names
 *
 * @param focusSearchBox  if called from the button click event, return to searchbox
 */
  onFindFilter: function(focusSearchBox)
  {
    let searchBox = document.getElementById("quickFilters-Search");
    let filterList = this.getFilterListElement();
    let keyWord = searchBox.value.toLocaleLowerCase();

    // simplest case: if filter was added or removed and searchbox is empty
    if (!keyWord && !focusSearchBox) {
      this.updateCountBox();
      return;
    }

    this.rebuildFilterList(gCurrentFilterList); // creates the unfiltered list
    if (!keyWord) {
      if (focusSearchBox)
        searchBox.focus();
      this.updateCountBox();
      return;
    }


    // rematch everything in the list, remove what doesn't match the search box
    let rows = this.getListElementCount(filterList);

    for(let i = rows - 1; i>=0; i--){
      let matched = true;
      var title;
       // SeaMonkey (Postbox) vs Thunderbird - treeview vs listbox
      if (filterList.nodeName === 'tree')
      {
        // SeaMonkey
        item = getFilter(i); // SM: defined in FilterListDialog.js (SM only)
        title = item.filterName;
        if(title.toLocaleLowerCase().indexOf(keyWord) === -1){
          matched = false;
          this.gFilterTreeView().performActionOnRow("delete", i);
          filterList.boxObject.invalidateRow(i);
          filterList.boxObject.rowCountChanged(i + 1, -1);
        }
      }
      else {
        // Thunderbird
        let item = filterList.getItemAtIndex(i);
        title = item.firstChild.getAttribute("label");
        if(title.toLocaleLowerCase().indexOf(keyWord) === -1)
        {
          matched = false;
          filterList.removeChild(item);
        }

      }
      if (matched)
        quickFilters.Util.logDebugOptional("filters", "matched filter: " + title);
    }
    this.updateCountBox();
    if (focusSearchBox)
      searchBox.focus();

  } ,


  validateFilterTargets: function(sourceURI, targetURI) {
    // fix any filters that might still point to the moved folder.

    // 1. nsIMsgAccountManager  loop through list of servers
    try {
      var Ci = Components.interfaces;
      var acctMgr = Components.classes["@mozilla.org/messenger/account-manager;1"]
                          .getService(Ci.nsIMsgAccountManager);
      var accounts = acctMgr.accounts;
      for (var i = 0; i < accounts.Count(); i++) {
        var account = accounts.QueryElementAt(i, Ci.nsIMsgAccount);
        if (account.incomingServer && account.incomingServer.canHaveFilters )
        {
          var ac = account.incomingServer.QueryInterface(Ci.nsIMsgIncomingServer);
          quickFilters.Util.logDebugOptional("filters", "checking account for filter changes: " +  ac.prettyName);
          // 2. getFilterList
          var filterList = ac.getFilterList(msgWindow).QueryInterface(Ci.nsIMsgFilterList);
          // 3. use  nsIMsgFilterList.matchOrChangeFilterTarget(oldUri, newUri, false)
          if (filterList) {
            filterList.matchOrChangeFilterTarget(sourceURI, targetURI, false)
          }
        }
      }
    }
    catch(ex) {
      quickFilters.Util.logException("Exception in quickFilters.List.validateFilterTargets ", ex);
    }

  }

};
